/* ========================================
 *
 *  Copyright Taiheisystem.co.,ltd., 
 *  All Rights Reserved
 *  File name : PDS*05
 *  Creator : T.Suzuki
 *  Date : 2018/10/15 ~ Start
 *
 * ========================================
 */
// メインコード
#include "project.h"
#include "stdio.h"

#define PROGRAM_NAME "PDS05 (prototype)"

void print(char *str)
{
    UART_1_PutString(str) ;
}

volatile int mode_sw = 0 ;
volatile int SW_Pressed_Flag = 0 ;
volatile int UP_Pressed_Flag = 0 ;
volatile int DOWN_Pressed_Flag = 0 ;
volatile int UP_Pressed_Duration = 0 ;
volatile int DOWN_Pressed_Duration = 0 ;

// UP ボタンの挙動定義
#define UP_DURATION_THRESHOLD 5
#define FAST_UP_STEP 0.1
#define SLOW_UP_STEP 0.01
// DOWN ボタンの挙動定義
#define DOWN_DURATION_THRESHOLD 5
#define FAST_DOWN_STEP 0.1
#define SLOW_DOWN_STEP 0.01
// 圧力の上限、下限定義 (根拠なし)
#define ABSOLUTE_UPPER_LIMIT 5.00
#define ABSOLUTE_LOWER_LIMIT 0.00

float upper_limit = ABSOLUTE_UPPER_LIMIT / 2.0 ; // 根拠なし、とりあえず入れてみました
float lower_limit = ABSOLUTE_UPPER_LIMIT / 2.0 ; // 根拠なし、とりあえず入れてみました

//ISR が割り込みであることの宣言
// CY_ISR_PROTO(SW_INT);
// SW_INT はピン名なので、sw_isr に変更しました。
CY_ISR_PROTO(sw_isr) ;
CY_ISR_PROTO(up_isr) ;
CY_ISR_PROTO(up_release_isr) ;
CY_ISR_PROTO(down_isr) ;
CY_ISR_PROTO(down_release_isr) ;

// 小さいほうの値を返すマクロ
#define MIN(x, y) ((x < y) ? x : y)

// 大きいほうの値を返すマクロ
#define MAX(x, y) ((x > y) ? x : y)

// UARTへの出力に使う文字列バッファのサイズ。
#define TRANSMIT_BUFFER_SIZE 80

// UARTの文字列バッファ。
char buf[TRANSMIT_BUFFER_SIZE];

// ADCの出力値（電圧）を格納する変数。小数値を格納したいのでfloat型(浮動小数点型)
float volt;

// 気圧の値。小数値を格納したいのでfloat型(浮動小数点型)
float pascal;

// 平均値（移動平均）。updateAvg()関数で更新する。
float avg = 0.0;

// 移動平均を更新するときの、瞬時値に対するの重み。
// これを小さくすると、滑らかに平均化されるが追従性が悪くなる。大きくすると、追従性はよくなるが瞬時値のブレの影響を受けやすくなる。
// UARTのデバッグ出力を見て、計測間隔に合わせて調整するとよい。
#define AVG_WEIGHT (0.5)

// 平均値を更新
float updateAvg(float val) {
    avg = avg * (1 - AVG_WEIGHT) + val * AVG_WEIGHT;
    return avg;
}

// 平均値を直接セット
void setAvg(float val) { avg = val; }

// 線形補間（上限、下限制限付き）
float lerp(float inMin, float inMax, float outMin, float outMax, float in) {
    float out = (in - inMin) * (outMax - outMin) / (inMax - inMin);
    return MIN(outMax, MAX(outMin, out));
}

// Connections:
//   P2.2 --- SW
//   P15.3--- UP
//   P15.1--- DOWN
//   P15.0--- HOS_SW
//   P15.4--- LED01
//   P3.6--- LED02
//   P3.5--- LED03
//   P2.1--- SWITCH_OUT_1
//   P2.7--- SWITCH_OUT_2
//   P12.6--- Rx_1
//   P12.7--- Tx_1
//   P3.3--- Vin_A
//   P3.4--- Vin_Ref
//   P3.7--- Vout_Ref
//   P0.0--- Pin_Com[0]
//   P0.1--- Pin_Com[1]
//   P0.2--- Pin_Com[2]
//   P0.3--- Pin_Com[3]
//   P15.5--- Pin_Seg[0]
//   P2.0--- Pin_Seg[1]
//   P1.2--- Pin_Seg[2]
//   P2.6--- Pin_Seg[3]
//   P1.4--- Pin_Seg[4]
//   P1.5--- Pin_Seg[5]
//   P1.6--- Pin_Seg[6]
//   P1.7--- Pin_Seg[7]

/* MODE
    0 : デフォルト状態（現状の圧力表示）
    1 : 定格圧力値の1/2で表示、UPとDOWNスイッチが使用可能
    2 : 定格圧力値の1/2で表示、UPとDOWNスイッチが使用可能
    3 : 表示部が全て消灯
    0 : デフォルトに戻る
*/

// このファイル内で定義している自前の関数のプロトタイプ宣言
float updateAvg(float val);
void setAvg(float val);
float lerp(float inMin, float inMax, float outMin, float outMax, float in);

// advanceStateMachineのプロトタイプ宣言
void advanceStateMachine(void) ;

void Mode0(void) ;
void Mode1(void) ;
void Mode2(void) ;
void Mode3(void) ;
void Mode4(void) ;
void Mode5(void) ;

void clear_duration(void) ;

void init_hardware(void)
{
    CyGlobalIntEnable;                                  /* Enable global interrupts */
// Mode Switch
    SW_INT_ClearPending() ;
    SW_INT_StartEx(sw_isr) ;
// UP Switch    
    UP_INT_ClearPending() ;
    UP_INT_StartEx(up_isr) ;
    UP_INT_Disable() ;
    UP_RELEASE_INT_ClearPending() ;
    UP_RELEASE_INT_StartEx(up_release_isr) ;
    UP_RELEASE_INT_Disable() ;
// DOWN Switch    
    DOWN_INT_ClearPending() ;
    DOWN_INT_StartEx(down_isr) ;
    DOWN_INT_Disable() ;
    DOWN_RELEASE_INT_ClearPending() ;
    DOWN_RELEASE_INT_StartEx(down_release_isr) ;
    DOWN_RELEASE_INT_Disable() ;
        
// 初期化/スタートアップ・コードを下記に記述する
    UART_1_Start();                                     /* UART_1 Start-up */
    sprintf(buf, "%s (%s %s)\r\n", PROGRAM_NAME, __DATE__, __TIME__) ;
    print(buf) ;

    LED_Driver_1_Start();                               /* 7Segment Start-up */
    
//    LED_Driver_1_WriteString7Seg("PSoC",0);             /* 7Segment */
    LED_Driver_1_ClearDisplayAll() ;
//    LED_Driver_1_Write7SegDigitHex(0x0A,0); 
//    LED_Driver_1_Write7SegDigitHex(0x0B,1); 
//    LED_Driver_1_Write7SegDigitHex(0x0C,2); 
//    LED_Driver_1_Write7SegDigitHex(0x0D,3);             /* 7Segment */
    
//    CyDelay(2000);                                      /* 2sec wait */
    
//    LED_Driver_1_PutChar7Seg( 0xFF,0 );                 /* 7Segment HEX表示 4桁 */
//    LED_Driver_1_PutChar7Seg( 0xFF,1 );	                /* 7Segment HEX表示 3桁 */
//	LED_Driver_1_PutChar7Seg( 0xFF,2 );	                /* 7Segment HEX表示 2桁 */
//	LED_Driver_1_PutChar7Seg( 0x7F,3 );             	/* 7Segment HEX表示 1桁 */
    
    ADC_DelSig_1_Start();                               /* AD変換 Start-up */
//    ADC_DelSig_1_StartConvert(); 
//    Timer_1_Start();                                    /* Timer_1 Start-up */
    Opamp_1_Start();                                    /* OPAMP Start-up */
    EEPROM_1_Start();                                   /* EEPROM Start-up */
//    SW_SetDriveMode(SW_DM_RES_UP);                      /* SW */
//    UP_SetDriveMode(UP_DM_RES_UP);                      /* UP */
//    DOWN_SetDriveMode(DOWN_DM_RES_UP);                  /* DOWN */
//    HOS_SW_SetDriveMode(HOS_SW_DM_RES_UP);              /* HOS_SW */
//    SWITCH_OUT_1_SetDriveMode(SWITCH_OUT_1_DM_STRONG);  /* Photo Relay 1 */
//    SWITCH_OUT_1_Write( 1 );                            /* Photo Relay 1 */
//    SWITCH_OUT_2_SetDriveMode(SWITCH_OUT_2_DM_STRONG);  /* Photo Relay 2 */
//    SWITCH_OUT_2_Write( 1 );                            /* Photo Relay 2 */
//    LED01_Write(1);
//    LED02_Write(1);
//    LED03_Write(1);
//    CyDelay(20);                                      /* 2sec wait */
//    LED01_Write(0);
//    LED02_Write(0);
//    LED03_Write(0);
   
    UART_1_PutString("ADC start\n");                    /* UART_1 */
}

/**
 * enable_up_down()
 * UP/DOWN ボタンを有効にする
 */
void enable_up_down(void)
{
    clear_duration() ;
    UP_INT_ClearPending() ;
    UP_INT_Enable() ;
    DOWN_INT_ClearPending() ;
    DOWN_INT_Enable() ;
}

/**
 * disable_up_down()
 * UP/DOWN ボタンを無効にする
　*/
void disable_up_down(void)
{
    UP_INT_Disable() ;
    UP_INT_ClearPending() ;
    DOWN_INT_Disable() ;
    DOWN_INT_ClearPending() ;
}

void measure(void)
{
    static uint8_t initialFlag = 1;                            /*  */
    ADC_DelSig_1_StartConvert(); 
    ADC_DelSig_1_IsEndConversion(ADC_DelSig_1_WAIT_FOR_RESULT) ;
    //       if (ADC_DelSig_1_IsEndConversion(ADC_DelSig_1_RETURN_STATUS)) {
    // ADCから電圧を取得。単位がmVなので、Vに変換するため1000.0で割る。その結果を変数voltに代入。
    volt = ADC_DelSig_1_CountsTo_mVolts(ADC_DelSig_1_GetResult16()) / 1000.0;
            
    // 0.8～4.0の値を、0.0～5.0の範囲に線形変換する。その結果を変数pascalに代入。
    pascal = lerp(0.8, 4.0, 0.0, 5.0, volt);

    if (initialFlag) {
    // 初回のみ、平均値（移動平均）を直接セットする。そうしないと、起動直後に0.0からゆっくり変化してしまってよくない。
        setAvg(pascal);
        initialFlag = 0;
    }
            
    // 平均値（移動平均）を更新。平均値は、上のほうで定義している変数avgとに代入される。
    updateAvg(pascal);
}

void display_pressure(void) 
{
    //  7セグLEDへ出力
    //  気圧の値を1000倍して、小数点第２位が１の位に来るようにする：0.12 -> 012
    //  LEDの出力開始位置：一番左(0)
    //  LEDの桁数：4
    //  表示方法：右寄せ、かつ、数字が４桁未満なら未使用桁に0を表示 例１：123->0123　例２：12 -> 0012
    LED_Driver_1_ClearDisplayAll() ;
    LED_Driver_1_Write7SegNumberDec(avg * 100.0 + 0.5, 0, 4, LED_Driver_1_ZERO_PAD);

    // 小数点を一番左から2の桁(1)に表示
    LED_Driver_1_PutDecimalPoint(1, 1);
            
    // UARTへデバッグ出力
    sprintf(buf, "ADC=%5.3f[V]  Raw=%5.3f[MPa] Average=%5.3f[MPa]\r\n", volt, pascal,avg);
    print(buf) ;    
}

void display_upper_limit(void)
{
    LED_Driver_1_ClearDisplayAll() ;
    LED_Driver_1_Write7SegNumberDec(upper_limit * 100.0 + 0.5, 0, 4, LED_Driver_1_ZERO_PAD);  
}

void display_lower_limit(void)
{
    LED_Driver_1_ClearDisplayAll() ;
    LED_Driver_1_Write7SegNumberDec(lower_limit * 100.0 + 0.5, 0, 4, LED_Driver_1_ZERO_PAD);    
}

//SWが0の時はモード0で現在の圧力値を表示
void doMode0(void)
{
    measure() ;
    display_pressure() ;
    CyDelay(1000) ; 
}

//SWが1の時はモード1で定格圧力値の1/2を表示かつUP、DOWNで上限値を設定   
void doMode1(void) 
{
    if (UP_Pressed_Flag) { // UPボタンが押されていた場合の処理
        if (UP_Pressed_Duration < UP_DURATION_THRESHOLD) {
            if (UP_Pressed_Duration == 0) { // 初回
                upper_limit += SLOW_UP_STEP ;
            }
            UP_Pressed_Duration++ ;
        } else {
            upper_limit += FAST_UP_STEP ;
        }
        if (upper_limit < lower_limit) { // 上限値が下限値を下回った場合
            upper_limit = lower_limit ;
        }
        print("mode 1: upper limit UP ") ;
    }
    if (DOWN_Pressed_Flag) { // DOWNボタンが押されていた場合の処理
        if (DOWN_Pressed_Duration < DOWN_DURATION_THRESHOLD) {
            if (DOWN_Pressed_Duration == 0) {
                upper_limit -= SLOW_DOWN_STEP ;
            }
            DOWN_Pressed_Duration++ ;
        } else {
            upper_limit -= FAST_DOWN_STEP ;
        }
        if (upper_limit < lower_limit) { // 上限値が下限値を下回った場合
            upper_limit = lower_limit ;
        }
        print("mode 1: upper limit DOWN ") ;
    }
    if (upper_limit >= ABSOLUTE_UPPER_LIMIT) {
        upper_limit = ABSOLUTE_UPPER_LIMIT ;
        if (UP_Pressed_Flag || DOWN_Pressed_Flag) {
            print(" = OVER = ") ;
        }
        LED01_Write(1) ; /* Turn ON LED01 */
    } else {
        LED01_Write(0) ; /* Turn OFF LED01 */
    }
    if (upper_limit <= ABSOLUTE_LOWER_LIMIT) {
        upper_limit = ABSOLUTE_LOWER_LIMIT ;
        if (UP_Pressed_Flag || DOWN_Pressed_Flag) {                
            print(" = UNDER = ") ;
        }
        LED02_Write(1) ; /* Turn ON LED02 */
    } else {
        LED02_Write(0) ; /* Turn OFF LED02 */
    }
    if (UP_Pressed_Flag || DOWN_Pressed_Flag) {
        sprintf(buf, "%02d.%02d\n\r", (int)(upper_limit), (int)(upper_limit * 100)%100) ;
        print(buf) ;
    }
    display_upper_limit() ;    
}

//SWが2の時はモード2で定格圧力値の1/2を表示かつUP、DOWNで下限値を設定、Photo Relay 1をON  
void doMode2(void) 
{
    // Photo Relay 1 ON
    if (UP_Pressed_Flag) { // UPボタンが押されていた場合の処理
        if (UP_Pressed_Duration < UP_DURATION_THRESHOLD) {
            if (UP_Pressed_Duration == 0) {
                lower_limit += SLOW_UP_STEP ;            
            }
            UP_Pressed_Duration++ ;
        } else {
            lower_limit += FAST_UP_STEP ;
        }
        if (lower_limit > upper_limit) {
            lower_limit = upper_limit ;
        }
        print("mode 2: lower limit UP ") ;
        UP_Pressed_Duration++ ;
    }
    if (DOWN_Pressed_Flag) { // DOWNボタンが押されていた場合の処理
        if (DOWN_Pressed_Duration < DOWN_DURATION_THRESHOLD) {
            if (DOWN_Pressed_Duration == 0) {
                lower_limit -= SLOW_DOWN_STEP ;         
            }
            DOWN_Pressed_Duration++ ;
        } else {
            lower_limit -= FAST_DOWN_STEP ;
        }
        print("mode 2: lower limit DOWN ") ;
    }
    if (lower_limit >= ABSOLUTE_UPPER_LIMIT) {
        lower_limit = ABSOLUTE_UPPER_LIMIT ;
        if (UP_Pressed_Flag || DOWN_Pressed_Flag) {   
            print(" = OVER = ") ;   
        }
        LED01_Write(1) ; /* Turn ON LED01 */
    } else {
        LED01_Write(0) ; /* Turn OFF LED01 */
    }
    if (lower_limit <= ABSOLUTE_LOWER_LIMIT) {
        lower_limit = ABSOLUTE_LOWER_LIMIT ;
        if (UP_Pressed_Flag || DOWN_Pressed_Flag) {   
            print(" = UNDER = ") ;
        }
        LED02_Write(1) ; /* Turn ON LED02 */
    } else {
        LED02_Write(0) ; /* Turn OFF LED02 */
    }
    if (UP_Pressed_Flag || DOWN_Pressed_Flag) {
        sprintf(buf, "%02d.%02d\n\r", (int)(lower_limit), (int)(lower_limit * 100)%100) ;
        print(buf) ;
    }
    display_lower_limit() ;    
}

//SWが3の時はモード3で定格圧力値の1/2を表示かつUP、DOWNで上限値を設定   
void doMode3(void)
{
    if (UP_Pressed_Flag) {
        print("mode 3: upper limit UP\n") ;
        UP_Pressed_Duration++ ;
    }
    if (DOWN_Pressed_Flag) {
        print("mode 3: upper limit DOWN\n") ;
        DOWN_Pressed_Duration++ ;
    }
    display_upper_limit() ;    
}

//SWが4の時はモード4で定格圧力値の1/2を表示かつUP、DOWNで下限値を設定、Photo Relay 2をON  
void doMode4(void)
{
    if (UP_Pressed_Flag) {
        print("mode 4: lower limit UP\n") ;
        UP_Pressed_Duration++ ;              
    }
    if (DOWN_Pressed_Flag) {
        print("mode 4: lower limit DOWN\n") ;
        DOWN_Pressed_Duration++ ;
     }           
    display_lower_limit() ;    
    // Photo Relay 2 ON
}

//SWが5の時はモード5で現在の圧力値を消灯（低消費モード）    
void doMode5(void)
{
    LED_Driver_1_ClearDisplayAll() ;    
}

int main(void) 
{
    init_hardware() ;

//　メイン・ループ    
    for (;;) {
        if (SW_Pressed_Flag) { // SWボタンが押されていた場合、状態遷移を行う
            advanceStateMachine() ;
            SW_Pressed_Flag = 0 ;
        }

        switch(mode_sw) {
        case 0: //SWが0の時はモード0で現在の圧力値を表示
            doMode0() ;
            break ;
        case 1: //SWが1の時はモード1で定格圧力値の1/2を表示かつUP、DOWNで上限値を設定 
            doMode1() ;
            break ;
        case 2: //SWが2の時はモード2で定格圧力値の1/2を表示かつUP、DOWNで下限値を設定、Photo Relay 1をON  
            doMode2() ;
            break ;
        case 3: //SWが3の時はモード3で定格圧力値の1/2を表示かつUP、DOWNで上限値を設定   
            doMode3() ;
            break ;
        case 4: //SWが4の時はモード4で定格圧力値の1/2を表示かつUP、DOWNで下限値を設定、Photo Relay 2をON  
            doMode4() ;
            break ;
        case 5: //SWが5の時はモード5で現在の圧力値を消灯（低消費モード）    
            doMode5() ;
            break ;
        default:
            sprintf(buf, "Unexpected Mode[%d] Returning to Mode 0\n\r", mode_sw) ;
            print(buf) ;
            mode_sw = 0 ;
            break ;
        }
        CyDelay(200) ; /* wait 0.2sec */
    }
}    
/* ----------------------------------------- */
/*        Device control / processing        */
/* ----------------------------------------- */           

CY_ISR(sw_isr)
{
    SW_INT_ClearPending();
    SW_Pressed_Flag = 1 ;
}

CY_ISR(up_isr)
{
    UP_INT_ClearPending() ;
    UP_Pressed_Flag = 1 ;
    UP_Pressed_Duration = 0 ;
    UP_RELEASE_INT_ClearPending() ;
    UP_RELEASE_INT_Enable() ;
}

CY_ISR(up_release_isr)
{
    UP_RELEASE_INT_Disable() ;
    UP_RELEASE_INT_ClearPending() ;
    UP_Pressed_Flag = 0 ;
}

CY_ISR(down_isr)
{
    DOWN_INT_ClearPending() ;
    DOWN_Pressed_Flag = 1 ;
    DOWN_Pressed_Duration = 0 ;
    DOWN_RELEASE_INT_ClearPending() ;
    DOWN_RELEASE_INT_Enable() ;
}

CY_ISR(down_release_isr)
{
    DOWN_RELEASE_INT_Disable() ;
    DOWN_RELEASE_INT_ClearPending() ;
    DOWN_Pressed_Flag = 0 ;
}

void clear_duration(void) 
{
    UP_Pressed_Duration = 0 ;
    DOWN_Pressed_Duration = 0 ;
}

/** 
 * advanceStateMachine()
 * 状態の遷移を行う。
 * mode_sw に移行するステートをアサインし、
 * 次ステートで最初に行うべき処理も行う。
 */
void advanceStateMachine(void)
{
    switch(mode_sw) {
    //SWが0の時はモード0で現在の圧力値を表示
        case 0:
            mode_sw = 1 ;
            enable_up_down() ;
            break ;
    // SWが1の時はモード1で定格圧力値の1/2を表示かつUP、DOWNで上限値を設定
        case 1:
            mode_sw = 2 ;
            enable_up_down() ;
            // Photo Relay 2 ON
            break ;
    // SWが2の時はモード2で定格圧力値の1/2を表示かつUP、DOWNで下限値を設定、Photo Relay 1をON    
        case 2:
            mode_sw = 3 ;
            enable_up_down() ;
            // Photo Relay 2 OFF?
            break ;
        // SWが3の時はモード3で定格圧力値の1/2を表示かつUP、DOWNで上限値を設定   
        case 3:
            mode_sw = 4 ;
            enable_up_down() ;
            // Photo Relay 2 ON
            break ;
        //SWが4の時はモード4で定格圧力値の1/2を表示かつUP、DOWNで下限値を設定、Photo Relay 2をON    
        case 4:
            mode_sw = 5 ;
            LED_Driver_1_ClearDisplayAll() ;
            disable_up_down() ; // モード5では UP/DOWN は無効
            // Photo Relay 2 OFF ?
            break ;
        //SWが5の時はモード5で現在の圧力値を消灯（低消費モード）    
        case 5:
            mode_sw = 0 ;
            disable_up_down() ; // モード0では UP/DOWN は無効
            break ;
        //SWが6の時はモード0で現在の圧力値を表示  
        default:
            sprintf(buf, "Unexpected SW MODE [%d] returning to Mode 0\r\n", mode_sw) ;
            print(buf) ;
            mode_sw = 0 ;
            disable_up_down() ;            
            break ;
    }            
    sprintf(buf, "SW MODE = %d\r\n", mode_sw) ;
    print(buf) ;
}
 
/* [] END OF FILE */
